@@ -11,12 +11,13 @@ import sys
 import time
 from base64 import b64encode
 from collections import deque
+from functools import partial
 from new_reader import reader
 from threefive import Stream, Cue
 
 MAJOR = "0"
 MINOR = "1"
-MAINTAINENCE = "21"
+MAINTAINENCE = "29"
 
 
 def version():
@@ -77,7 +78,8 @@ class SCTE35:
         """
         mk_cue_tag
         """
-        return f'#EXT-X-SCTE35:CUE="{b64encode(cue.bites).decode()}"'
+        if cue:
+            return f'#EXT-X-SCTE35:CUE="{b64encode(cue.bites).decode()}"'
 
     @staticmethod
     def is_cue_out(cue):
@@ -145,7 +147,7 @@ class X9K3(Stream):
         self.delete = False
         self.seg = SegData()
         self.sidecar = None
-        self.iframes_only = False
+        self.replay = False
         self._parse_args()
 
     def _parse_args(self):
@@ -175,8 +177,7 @@ class X9K3(Stream):
             "-s",
             "--sidecar",
             default=None,
-            help="""sidecar file of scte35 cues. each line contains  (PTS, CueString)
-                        Example:  89718.451333, /DARAAAAAAAAAP/wAAAAAHpPv/8=""",
+            help="""sidecar file of scte35 cues. each line contains  PTS, Cue""",
         )
 
         parser.add_argument(
@@ -194,32 +195,68 @@ class X9K3(Stream):
             action="store_const",
             default=False,
             const=True,
-            help="delete segments ( enables live mode )",
+            help="delete segments ( enables --live )",
         )
+
+        parser.add_argument(
+            "-r",
+            "--replay",
+            action="store_const",
+            default=False,
+            const=True,
+            help="Flag for replay (looping) ( enables --live and --delete )",
+        )
+
+        parser.add_argument(
+            "-v",
+            "--version",
+            action="store_const",
+            default=False,
+            const=True,
+            help="Show version",
+        )
+
         args = parser.parse_args()
         self._apply_args(args)
 
+    def _args_version(self, args):
+        if args.version:
+            print(version())
+            sys.exit()
+
+    def _args_input(self, args):
+        if args.input:
+            self._tsdata = args.input
+        else:
+            self._tsdata = sys.stdin.buffer
+
+    def _args_output_dir(self, args):
+        self.output_dir = args.output_dir
+        if not os.path.isdir(args.output_dir):
+            os.mkdir(args.output_dir)
+
+    def _args_flags(self, args):
+        if args.live or args.delete or args.replay:
+            self.live = True
+            if args.delete or args.replay:
+                self.delete = True
+                if args.replay:
+                    self.replay = True
+
+    def _args_sidecar(self, args):
+        if args.sidecar:
+            self.load_sidecar(args.sidecar)
+
     def _apply_args(self, args):
         """
         _apply_args  uses command line args
         to set X9K3 instance vars
         """
-        if args.input:
-            self._tsdata = args.input
-        else:
-            self._tsdata = sys.stdin.buffer
-
-        self.output_dir = args.output_dir
-        if not os.path.isdir(args.output_dir):
-            os.mkdir(args.output_dir)
-
-        self.live = args.live
-        self.delete = args.delete
-        if args.delete:
-            self.live = True
-
-        if args.sidecar:
-            self.load_sidecar(args.sidecar)
+        self._args_version(args)
+        self._args_input(args)
+        self._args_output_dir(args)
+        self._args_flags(args)
+        self._args_sidecar(args)
         if isinstance(self._tsdata, str):
             self._tsdata = reader(self._tsdata)
 
@@ -274,10 +311,11 @@ class X9K3(Stream):
         for the next sidecar cue and inserts the cue if needed.
         """
         if self.sidecar:
-            if self.sidecar[0][0] < self.pid2pts(pid):
+            if self.sidecar[0][0] <= self.pid2pts(pid):
                 raw = self.sidecar.popleft()[1]
                 self.scte35.cue = Cue(raw)
                 self.scte35.cue.decode()
+                self.scte35.cue_tag = self.scte35.mk_cue_tag(self.scte35.cue)
                 self._chk_cue(pid)
 
     def chk_stream_cues(self, pkt, pid):
@@ -285,12 +323,15 @@ class X9K3(Stream):
         chk_stream_cues checks scte35 packets
         and inserts the cue.
         """
-        self.scte35.cue = self._parse_scte35(pkt, pid)
-        if self.scte35.cue:
+        cue = self._parse_scte35(pkt, pid)
+        if cue:
+            self.scte35.cue = cue
             self._chk_cue(pid)
 
     def _add_discontinuity(self):
         self.active_data.write("#EXT-X-DISCONTINUITY\n")
+        self.seg.seg_start = None
+        self.seg.seg_stop = None
 
     def _chk_cue(self, pid):
         """
@@ -320,15 +361,16 @@ class X9K3(Stream):
         _mk_cue_splice_point inserts a tag
         at the time specified in the cue.
         """
-        self.scte35.cue_tag = self.scte35.mk_cue_tag(self.scte35.cue)
-        self._add_discontinuity()
-        print(f"Splice Point {self.scte35.cue.command.name}@{self.scte35.cue_time}")
-        self.active_data.write(f"# Splice Point @ {self.scte35.cue_time}\n")
-        self.scte35.cue_out_cue_in()
-        if self.scte35.cue_out is None:
-            self.scte35.cue_time = None
-        self.active_data.write(self.scte35.cue_tag + "\n")
-        self.scte35.cue_tag = None
+        if self.scte35.cue:
+            self.scte35.cue_tag = self.scte35.mk_cue_tag(self.scte35.cue)
+            self._add_discontinuity()
+            print(f"Splice Point {self.scte35.cue.command.name}@{self.scte35.cue_time}")
+            self.active_data.write(f"# Splice Point @ {self.scte35.cue_time}\n")
+            self.scte35.cue_out_cue_in()
+            if self.scte35.cue_out is None:
+                self.scte35.cue_time = None
+            self.active_data.write(self.scte35.cue_tag + "\n")
+            self.scte35.cue_tag = None
 
     def cue_out_continue(self):
         """
@@ -336,7 +378,6 @@ class X9K3(Stream):
         if there is an active SCTE35 cue,
         the live sliding window of segments
         has a tag with CUE-OUT=CONT
-
         """
         if self.window_slot > self.WINDOW_SLOTS:
             self.window_slot = 0
@@ -349,16 +390,17 @@ class X9K3(Stream):
         """
         _mk_segment cuts hls segments
         """
+        now = self.pid2pts(pid)
         if self.scte35.cue_time:
-            if self.seg.seg_start < self.scte35.cue_time < self.seg.seg_stop:
+            if self.seg.seg_start < self.scte35.cue_time < now:
                 self.seg.seg_stop = self.scte35.cue_time
                 self._mk_cue_splice_point()
                 self.scte35.cue_time = None
-        now = self.pid2pts(pid)
-        if now >= self.seg.seg_stop:
-            self.seg.seg_stop = now
-            self._write_segment()
-            self._write_manifest()
+        if self.seg.seg_stop:
+            if now >= self.seg.seg_stop:
+                self.seg.seg_stop = now
+                self._write_segment()
+                self._write_manifest()
 
     def _write_segment(self):
         """
@@ -479,12 +521,13 @@ class X9K3(Stream):
 
         if _nal(pkt):
             return True
-        if self._afc_flag(pkt):
+        if not self._afc_flag(pkt):
+            return False
+        if self._pcr_flag(pkt):
             if _rai_flag(pkt):
                 return True
-            if _abc_flags(pkt):
-                return True
-
+        if _abc_flags(pkt):
+            return True
         return False
 
     @staticmethod
@@ -538,25 +581,51 @@ class X9K3(Stream):
         if self.start:
             self.active_segment.write(pkt)
 
+    def loop(self):
+        """
+        loop  loops a video in the hls manifest.
+        sliding window and throttled to simulate live playback,
+        segments are deleted when they fall out the sliding window.
+        """
+        if not self._find_start():
+            return False
+        _ = [
+            self._parse(pkt)
+            for pkt in iter(partial(self._tsdata.read, self._PACKET_SIZE), b"")
+        ]
+        self._add_discontinuity()
+        self._tsdata.seek(0)
+        self.loop()
+
+    def run(self):
+        """
+        run calls replay() if replay is set
+        or else it calls decode()
+        """
+        if self.replay:
+            self.loop()
+        else:
+            self.decode()
+
 
 def cli():
     """
     cli provides one function call
     for running X9K3  with command line args
+    usage: x9k3 [-h] [-i INPUT] [-o OUTPUT_DIR] [-s SIDECAR] [-l] [-d] [-r] [-v]
 
+    optional arguments:
       -h, --help            show this help message and exit
       -i INPUT, --input INPUT
-                        Input source, like "/home/a/vid.ts" or
-                        "udp://@235.35.3.5:3535" or "https://futzu.com/xaa.ts"
+                            Input source, like "/home/a/vid.ts" or "udp://@235.35.3.5:3535" or "https://futzu.com/xaa.ts"
       -o OUTPUT_DIR, --output_dir OUTPUT_DIR
-                        Directory for segments and index.m3u8 ( created if it
-                        does not exist )
+                            Directory for segments and index.m3u8 ( created if it does not exist )
       -s SIDECAR, --sidecar SIDECAR
-                        sidecar file of scte35 cues. each line contains (PTS,
-                        CueString) Example: 89718.451333,
-                        /DARAAAAAAAAAP/wAAAAAHpPv/8=
+                            sidecar file of scte35 cues. each line contains PTS, Cue
       -l, --live            Flag for a live event ( enables sliding window m3u8 )
-      -d, --delete          delete segments ( enables live mode )
+      -d, --delete          delete segments ( enables --live )
+      -r, --replay          Flag for replay (looping) ( enables --live and --delete )
+      -v, --version         Show version
 
     Two lines of code gives you a full X9K3 command line tool.
 
@@ -564,9 +633,10 @@ def cli():
      cli()
 
     """
-    x9k3 = X9K3()
-    x9k3.decode()
+    x9 = X9K3()
+    x9.run()
 
 
 if __name__ == "__main__":
-    cli()
+    x9k = X9K3()
+    x9k.run()
